@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2025 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -72,7 +72,9 @@ Sample::Sample( const QString& filepath, const License& license, int frames, int
 	__is_modified( false ),
 	m_license( license )
 {
-	assert( filepath.lastIndexOf( "/" ) >0 );
+	if ( filepath.lastIndexOf( "/" ) <= 0 ) {
+		WARNINGLOG( QString( "Provided filepath [%1] does not seem like an absolute path. Sample will most probably be unable to load." ) );
+	}
 }
 
 Sample::Sample( std::shared_ptr<Sample> pOther ): Object( *pOther ),
@@ -157,9 +159,28 @@ bool Sample::load( float fBpm )
 	SF_INFO sound_info = {0};
 
 	// Opens file in read-only mode.
-	SNDFILE* file = sf_open( get_filepath().toLocal8Bit(), SFM_READ, &sound_info );
-	if ( !file ) {
-		ERRORLOG( QString( "[Sample::load] Error loading file %1" ).arg( get_filepath() ) );
+#ifdef WIN32
+	// On Windows we use a special version of sf_open to ensure we get all
+	// characters of the filename entered in the GUI right. No matter which
+	// encoding was used locally.
+	// We have to terminate the string using a null character ourselves.
+	QString sPaddedPath = get_filepath().append( '\0' );
+	wchar_t* encodedFilename = new wchar_t[ sPaddedPath.size() ];
+
+	sPaddedPath.toWCharArray( encodedFilename );
+
+	SNDFILE* file = sf_wchar_open( encodedFilename, SFM_READ,
+								   &sound_info );
+	delete encodedFilename;
+#else
+	SNDFILE* file = sf_open( get_filepath().toLocal8Bit(), SFM_READ,
+							 &sound_info );
+#endif
+	if ( file == nullptr ) {
+		ERRORLOG( QString( "Error loading file [%1] with format [%2]: %3" )
+				  .arg( get_filepath() )
+				  .arg( sndfileFormatToQString( sound_info.format ) )
+				  .arg( sf_strerror( file ) ) );
 		return false;
 	}
 	
@@ -698,15 +719,34 @@ bool Sample::write( const QString& path, int format )
 	sf_info.samplerate = __sample_rate;
 	sf_info.format = format;
 	if ( !sf_format_check( &sf_info ) ) {
-		___ERRORLOG( "SF_INFO error" );
+		ERRORLOG( "SF_INFO error" );
 		delete[] obuf;
 		return false;
 	}
 
-	SNDFILE* sf_file = sf_open( path.toLocal8Bit().data(), SFM_WRITE, &sf_info ) ;
+#ifdef WIN32
+	// On Windows we use a special version of sf_open to ensure we get all
+	// characters of the filename entered in the GUI right. No matter which
+	// encoding was used locally.
+	// We have to terminate the string using a null character ourselves.
+	QString sPaddedPath = QString( path ).append( '\0' );
+	wchar_t* encodedFilename = new wchar_t[ sPaddedPath.size() ];
 
-	if ( sf_file==nullptr ) {
-		___ERRORLOG( QString( "sf_open error : %1" ).arg( sf_strerror( sf_file ) ) );
+	sPaddedPath.toWCharArray( encodedFilename );
+	
+	SNDFILE* sf_file = sf_wchar_open( encodedFilename, SFM_WRITE,
+								   &sf_info );
+	delete encodedFilename;
+#else
+	const auto sPathLocal8Bit = path.toLocal8Bit();
+	SNDFILE* sf_file = sf_open( sPathLocal8Bit.data(), SFM_WRITE, &sf_info );
+#endif
+
+	if ( sf_file == nullptr ) {
+		ERRORLOG( QString( "Unable to create file [%1] with format [%2]: %3" )
+				  .arg( path )
+				  .arg( sndfileFormatToQString( format ) )
+				  .arg( sf_strerror( sf_file ) ) );
 		sf_close( sf_file );
 		delete[] obuf;
 		return false;
@@ -715,7 +755,7 @@ bool Sample::write( const QString& path, int format )
 	sf_count_t res = sf_writef_float( sf_file, obuf, __frames );
 
 	if ( res<=0 ) {
-		___ERRORLOG( QString( "sf_writef_float error : %1" ).arg( sf_strerror( sf_file ) ) );
+		ERRORLOG( QString( "sf_writef_float error : %1" ).arg( sf_strerror( sf_file ) ) );
 		sf_close( sf_file );
 		delete[] obuf;
 		return false;
@@ -793,6 +833,158 @@ QString Sample::toQString( const QString& sPrefix, bool bShort ) const {
 	return sOutput;
 }
 
+QString Sample::sndfileFormatToQString( int nFormat ) {
+	QString sFormat;
+	if ( nFormat & 0x010000 ) {
+		sFormat = "Microsoft WAV format (little endian)";
+	} else if ( nFormat & 0x020000 ) {
+		sFormat = "Apple/SGI AIFF format (big endian)";
+	} else if ( nFormat & 0x030000 ) {
+		sFormat = "Sun/NeXT AU format (big endian)";
+	} else if ( nFormat & 0x040000 ) {
+		sFormat = "RAW PCM data";
+	} else if ( nFormat & 0x050000 ) {
+		sFormat = "Ensoniq PARIS file format";
+	} else if ( nFormat & 0x060000 ) {
+		sFormat = "Amiga IFF / SVX8 / SV16 format";
+	} else if ( nFormat & 0x070000 ) {
+		sFormat = "Sphere NIST format";
+	} else if ( nFormat & 0x080000 ) {
+		sFormat = "VOC files";
+	} else if ( nFormat & 0x0A0000 ) {
+		sFormat = "Berkeley/IRCAM/CARL";
+	} else if ( nFormat & 0x0B0000 ) {
+		sFormat = "Sonic Foundry's 64 bit RIFF/WAV";
+	} else if ( nFormat & 0x0C0000 ) {
+		sFormat = "Matlab (tm) V4.2 / GNU Octave 2.0";
+	} else if ( nFormat & 0x0D0000 ) {
+		sFormat = "Matlab (tm) V5.0 / GNU Octave 2.1";
+	} else if ( nFormat & 0x0E0000 ) {
+		sFormat = "Portable Voice Format";
+	} else if ( nFormat & 0x0F0000 ) {
+		sFormat = "Fasttracker 2 Extended Instrument";
+	} else if ( nFormat & 0x100000 ) {
+		sFormat = "HMM Tool Kit format";
+	} else if ( nFormat & 0x110000 ) {
+		sFormat = "Midi Sample Dump Standard";
+	} else if ( nFormat & 0x120000 ) {
+		sFormat = "Audio Visual Research";
+	} else if ( nFormat & 0x130000 ) {
+		sFormat = "MS WAVE with WAVEFORMATEX";
+	} else if ( nFormat & 0x160000 ) {
+		sFormat = "Sound Designer 2";
+	} else if ( nFormat & 0x170000 ) {
+		sFormat = "FLAC lossless file format";
+	} else if ( nFormat & 0x180000 ) {
+		sFormat = "Core Audio File format";
+	} else if ( nFormat & 0x190000 ) {
+		sFormat = "Psion WVE format";
+	} else if ( nFormat & 0x200000 ) {
+		sFormat = "Xiph OGG container";
+	} else if ( nFormat & 0x210000 ) {
+		sFormat = "Akai MPC 2000 sampler";
+	} else if ( nFormat & 0x220000 ) {
+		sFormat = "RF64 WAV file";
+	} else if ( nFormat & 0x230000 ) {
+		sFormat = "MPEG-1/2 audio stream_FORMAT_OGG";
+	} else {
+		return QString( "Unknown format [%1]" ).arg( nFormat );
+	}
+
+	QString sSubType;
+	if ( nFormat & 0x0001 ) {
+		sSubType = "Signed 8 bit data";
+	} else if ( nFormat & 0x0002 ) {
+		sSubType = "Signed 16 bit data";
+	} else if ( nFormat & 0x0003 ) {
+		sSubType = "Signed 24 bit data";
+	} else if ( nFormat & 0x0004 ) {
+		sSubType = "Signed 32 bit data";
+	} else if ( nFormat & 0x0005 ) {
+		sSubType = "Unsigned 8 bit data (WAV and RAW only)";
+	} else if ( nFormat & 0x0006 ) {
+		sSubType = "32 bit float data";
+	} else if ( nFormat & 0x0007 ) {
+		sSubType = "64 bit float data";
+	} else if ( nFormat & 0x0010 ) {
+		sSubType = "U-Law encoded";
+	} else if ( nFormat & 0x0011 ) {
+		sSubType = "A-Law encoded";
+	} else if ( nFormat & 0x0012 ) {
+		sSubType = "IMA ADPCM";
+	} else if ( nFormat & 0x0013 ) {
+		sSubType = "Microsoft ADPCM";
+	} else if ( nFormat & 0x0020 ) {
+		sSubType = "GSM 6.10 encoding";
+	} else if ( nFormat & 0x0021 ) {
+		sSubType = "OKI / Dialogix ADPCM";
+	} else if ( nFormat & 0x0022 ) {
+		sSubType = "16kbs NMS G721-variant encoding";
+	} else if ( nFormat & 0x0023 ) {
+		sSubType = "24kbs NMS G721-variant encoding";
+	} else if ( nFormat & 0x0024 ) {
+		sSubType = "32kbs NMS G721-variant encoding";
+	} else if ( nFormat & 0x0030 ) {
+		sSubType = "32kbs G721 ADPCM encoding";
+	} else if ( nFormat & 0x0031 ) {
+		sSubType = "24kbs G723 ADPCM encoding";
+	} else if ( nFormat & 0x0032 ) {
+		sSubType = "40kbs G723 ADPCM encoding";
+	} else if ( nFormat & 0x0040 ) {
+		sSubType = "12 bit Delta Width Variable Word encoding";
+	} else if ( nFormat & 0x0041 ) {
+		sSubType = "16 bit Delta Width Variable Word encoding";
+	} else if ( nFormat & 0x0042 ) {
+		sSubType = "24 bit Delta Width Variable Word encoding";
+	} else if ( nFormat & 0x0043 ) {
+		sSubType = "N bit Delta Width Variable Word encoding";
+	} else if ( nFormat & 0x0050 ) {
+		sSubType = "8 bit differential PCM (XI only)";
+	} else if ( nFormat & 0x0051 ) {
+		sSubType = "16 bit differential PCM (XI only)";
+	} else if ( nFormat & 0x0060 ) {
+		sSubType = "Xiph Vorbis encoding";
+	} else if ( nFormat & 0x0064 ) {
+		sSubType = "Xiph/Skype Opus encoding";
+	} else if ( nFormat & 0x0070 ) {
+		sSubType = "Apple Lossless Audio Codec (16 bit)";
+	} else if ( nFormat & 0x0071 ) {
+		sSubType = "Apple Lossless Audio Codec (20 bit)";
+	} else if ( nFormat & 0x0072 ) {
+		sSubType = "Apple Lossless Audio Codec (24 bit)";
+	} else if ( nFormat & 0x0073 ) {
+		sSubType = "Apple Lossless Audio Codec (32 bit)";
+	} else if ( nFormat & 0x0080 ) {
+		sSubType = "MPEG-1 Audio Layer I";
+	} else if ( nFormat & 0x0081 ) {
+		sSubType = "MPEG-1 Audio Layer II";
+	} else if ( nFormat & 0x0082 ) {
+		sSubType = "MPEG-2 Audio Layer III";
+	} else {
+		INFOLOG( QString( "Unknown subtype [%1]" ).arg( nFormat ) );
+	}
+
+	QString sEndianness;
+	if ( nFormat & 0x00000000 ) {
+		sEndianness = "Default file endian-ness";
+	} else if ( nFormat & 0x10000000 ) {
+		sEndianness = "Force little endian-ness";
+	} else if ( nFormat & 0x20000000 ) {
+		sEndianness = "Force big endian-ness";
+	} else if ( nFormat & 0x30000000 ) {
+		sEndianness = "Force CPU endian-ness";
+	}
+	
+	if ( ! sSubType.isEmpty() ) {
+		sFormat.append( QString( " - %1" ).arg( sSubType ) );
+	}
+	if ( ! sEndianness.isEmpty() ) {
+		sFormat.append( QString( " - %1" ).arg( sEndianness ) );
+	}
+
+	return sFormat;
+}
+
 #ifdef H2CORE_HAVE_RUBBERBAND
 static double compute_pitch_scale( const Sample::Rubberband& rb )
 {
